@@ -5,8 +5,10 @@ import socket
 import json
 import yaml
 import sys
+import re
+import ssl
 
-from datetime import datetime
+from datetime import datetime, timezone
 
 # python3 -m pip dnslib
 import dnslib
@@ -52,8 +54,10 @@ async def cb_ondnstap(dnstap_decoder, payload, tcp_writer, cfg):
     dnstap_decoder.parse_from_bytes(payload)
     dm = dnstap_decoder.message
     
-    if "dnstap-identities" in cfg["input-mode"]:
-        if dnstap_decoder.identity.decode() not in cfg["input-mode"]["dnstap-identities"]:
+    # filtering by dnstap identity ?
+    if cfg["filter"]["dnstap-identities"] is not None:
+        if re.match(cfg["filter"]["dnstap-identities"], dnstap_decoder.identity.decode()) is None:
+            del dm
             return
             
     tap = { "identity": dnstap_decoder.identity.decode(),
@@ -79,25 +83,27 @@ async def cb_ondnstap(dnstap_decoder, payload, tcp_writer, cfg):
     if (dm.type.value % 2 ) == 1 :
         dnstap_parsed = dnslib.DNSRecord.parse(dm.query_message)
         tap["length"] = len(dm.query_message)
-        query_time_milli = (round(dm.query_time_nsec / 1000000) / 1000)
-        d1 = dm.query_time_sec +  query_time_milli
-        tap["timestamp"] = datetime.fromtimestamp(d1).strftime(DATETIME_FORMAT)[:-3]
+        d1 = dm.query_time_sec +  (round(dm.query_time_nsec ) / 1000000000)
+        tap["timestamp"] = datetime.fromtimestamp(d1, tz=timezone.utc).isoformat()
         
     # handle response message
     if (dm.type.value % 2 ) == 0 :
         dnstap_parsed = dnslib.DNSRecord.parse(dm.response_message)
         tap["length"] = len(dm.response_message)
-
-        reply_time_milli = (round(dm.response_time_nsec / 1000000) / 1000) 
-        d2 = dm.response_time_sec + reply_time_milli
-        tap["timestamp"] = datetime.fromtimestamp(d2).strftime(DATETIME_FORMAT)[:-3]
-
+        d2 = dm.response_time_sec + (round(dm.response_time_nsec ) / 1000000000) 
+        tap["timestamp"] = datetime.fromtimestamp(d2, tz=timezone.utc).isoformat()
+        
     # common params
     if len(dnstap_parsed.questions):
         tap["query-name"] = str(dnstap_parsed.questions[0].get_qname())
         tap["query-type"] = dnslib.QTYPE[dnstap_parsed.questions[0].qtype]
     tap["code"] = dnslib.RCODE[dnstap_parsed.header.rcode]
     
+    # filtering by qname ?
+    if cfg["filter"]["qname-regex"] is not None:
+        if re.match(cfg["filter"]["qname-regex"], tap["query-name"]) is None:
+            del dm; del tap;
+            return
     
     # reformat dnstap message
     if cfg["output-format"]["text"]:
@@ -229,11 +235,18 @@ def start_receiver():
                                                   loop=loop)
     # asynchronous tcp socket
     else:
+        ssl_context = None
+        if cfg["input-mode"]["tls-support"]:
+            ssl_context = ssl.create_default_context(ssl.Purpose.CLIENT_AUTH)
+            ssl_context.load_cert_chain(certfile=cfg["input-mode"]["tls-server-cert"], 
+                                        keyfile=cfg["input-mode"]["tls-server-key"])
+        
         logging.debug("Listening on %s:%s" % (cfg["input-mode"]["local-address"],
                                               cfg["input-mode"]["local-port"])), 
         socket_server = asyncio.start_server(lambda r, w: cb_onconnect(r, w, cfg),
                                              cfg["input-mode"]["local-address"],
                                              cfg["input-mode"]["local-port"],
+                                             ssl=ssl_context,
                                              loop=loop)
 
     # run until complete
